#!/bin/sh

config() {
  NEW="$1"
  OLD="`dirname $NEW`/`basename $NEW .new`"
  # If there's no config file by that name, mv it over:
  if [ ! -r $OLD ]; then
    mv $NEW $OLD
  elif [ "`cat $OLD | md5sum`" = "`cat $NEW | md5sum`" ]; then # toss the redundant copy
    rm $NEW
  fi
  # Otherwise, we leave the .new copy for the admin to consider...
}
config etc/uniconf.conf.new
( cd usr/lib ; rm -rf libwvstreams.so )
( cd usr/lib ; ln -sf libwvstreams.so.4.4 libwvstreams.so )
( cd usr/lib ; rm -rf libwvqt.so )
( cd usr/lib ; ln -sf libwvqt.so.4.4 libwvqt.so )
( cd usr/lib ; rm -rf libuniconf.so )
( cd usr/lib ; ln -sf libuniconf.so.4.4 libuniconf.so )
( cd usr/lib ; rm -rf libwvutils.so )
( cd usr/lib ; ln -sf libwvutils.so.4.4 libwvutils.so )
( cd usr/lib ; rm -rf libwvbase.so )
( cd usr/lib ; ln -sf libwvbase.so.4.4 libwvbase.so )
